# 転置式ファイル暗号の解読
# https://www.nostarch.com/crackingcodes (BSD Licensed)

import time, os, sys, transpositionDecrypt, detectEnglish

inputFilename = 'frankenstein.encrypted.txt'
outputFilename = 'frankenstein.decrypted.txt'

def main():
    if not os.path.exists(inputFilename):
        print('The file %s does not exist. Quitting.' % (inputFilename))
        sys.exit()

    inputFile = open(inputFilename)
    content = inputFile.read()
    inputFile.close()

    hackedMessage = hackTransposition(content)

    if hackedMessage != None:
        print('Writing decrypted text to %s.' % (outputFilename))

        outputFile = open(outputFilename, 'w')
        outputFile.write(hackedMessage)
        outputFile.close()
    else:
        print('Failed to hack encryption.')


# hackTransposition()関数のコードはtranspositionHacker.pyをコピー＆ペーストし、
# 若干の修正を加えた.
def hackTransposition(message):
    print('Hacking...')
    # PythonプログラムはCtrl-C（Windowsの場合）あるいは
    # Ctrl-D（macOSやLinuxの場合）を押していつでも停止できる.
    print('(Press Ctrl-C or Ctrl-D to quit at any time.)')

    for key in range(1, len(message)):
        print('Trying key #%s... ' % (key), end='')
        sys.stdout.flush()

        # 1つの鍵をテストする時間を計測したいので、
        # 開始時間startTimeを保持する.
        startTime = time.time()

        decryptedText = transpositionDecrypt.decryptMessage(key, message)
        englishPercentage = round(detectEnglish.getEnglishCount(decryptedText) * 100, 2)

        totalTime = round(time.time() - startTime, 3)
        print('Test time: %s seconds, ' % (totalTime), end='')
        sys.stdout.flush() # 画面にフラッシュする（バッファ内のデータを出力する）.

        print('Percent English: %s%%' % (englishPercentage))
        if englishPercentage > 20:
            print()
            print('Key ' + str(key) + ': ' + decryptedText[:100])
            print()
            print('Enter D if done, anything else to continue hacking:')
            response = input('> ')
            if response.strip().upper().startswith('D'):
                return decryptedText
    return None


# （モジュールとしてインポートされる代わりに）transpositionFileHacker.pyを実行すると、
# main()関数を呼び出す.
if __name__ == '__main__':
    main()